from sys import path
from os import getcwd

from black_mamba.deploy import DeployContract
from black_mamba.auth import Authentication


path.append(getcwd())
import settings
auth = settings.auth
development_auth = auth["development"]

keyfile = development_auth["keyfile"]
password = development_auth["password"]

mnemonic = "coral allow abandon recipe top tray caught video climb similar prepare bracket antenna rubber announce gauge volume hub hood burden skill immense add acid"
private_key = Authentication.get_private_key_from_mnemonic(mnemonic)

# private_key = Authentication.get_private_key_from_keyfile_by_explicit_password(keyfile, password)
# private_key = Authentication.get_private_key_from_keyfile_by_asking_password(keyfile)
# private_key = development_auth["private_key"]

deploy_contract_instance = DeployContract()
parameters = []
tx_params = {{ "from": "" }}

deploy_contract_instance.deploy_contract("{smart_contract_name}", parameters, tx_params, private_key)



import requests

from black_mamba import settings_directory
import settings

etherscan = settings.etherscan

api_key = etherscan["development"]["api_key"]

server = 'https://api.etherscan.io'


def erc20_total_supply(contract_address):
    """Getting the total supply of ERC-20 token

    Parameters
    ----------
    contract_address : str
      The address of the ERC-20 token smart contract
    """
    response = requests.get(f'{server}/api?module=stats&action=tokensupply&contractaddress={contract_address}&apikey={api_key}')
    if response.ok:
        return response.json()["result"]

def erc20_account_balance(contract_address, account_address):
    """Getting the balance of ERC-20 token of an account

    Parameters
    ----------
    contract_address : str
      The address of the ERC-20 token smart contract
    account_address : str
      The address of the account
    """
    response = requests.get(f'{server}/api?module=account&action=tokenbalance&contractaddress={contract_address}&address={account_address}&tag=latest&apikey={api_key}')
    if response.ok:
        return response.json()["result"]



from pathlib import Path
from shutil import rmtree, copy


class TestWithContracts():

    tmp_path = Path('/tmp/mamba_tests/') # TODO: Change this into dynamic temp directory

    def setup_method(self, method):
        self.tmp_path.mkdir()
        self.fixtures_dir = Path('.') / Path('test') / Path('fixtures')
        self.build_dir = self.tmp_path / Path('build')
        self.migrations_dir = self.tmp_path / Path("migrations")
        self.decentralized_app_dir = self.tmp_path / Path("decentralized_app")
        self.contracts_dir = self.tmp_path / Path("contracts")
        self.build_contracts_dir = self.tmp_path / Path('build') / Path('contracts')
        self.json_compiled_fixture = self.fixtures_dir / Path("HelloWorld.json")
        self.json_compiled_file = self.build_contracts_dir / Path('HelloWorld.json')
        self.solidity_json_compiled_file = self.build_contracts_dir / Path('SimpleStorage.json')
        self.migration_file = self.migrations_dir / Path("deploy_HelloWorld.py")
        self.solidity_migration_file = self.migrations_dir / Path("deploy_SimpleStorage.py")
        self.migration2_file = self.migrations_dir / Path("deploy_HelloParameters.py")
        self.json2_compiled_fixture = self.fixtures_dir / Path("HelloParameters.json")
        self.json2_compiled_file = self.build_contracts_dir / Path('HelloParameters.json')
        self.keyfile2_file = self.tmp_path / Path('keyfile2.json')
        self.ethpm_build_dir = self.tmp_path / Path("ethpm_build")
        self.manifest_file = self.ethpm_build_dir / Path("manifest.json")

        if not self.build_dir.exists():
            self.build_dir.mkdir()
        if not self.build_contracts_dir.exists():
            self.build_contracts_dir.mkdir()
        if not self.migrations_dir.exists():
            self.migrations_dir.mkdir()
        if not self.decentralized_app_dir.exists():
            self.decentralized_app_dir.mkdir()
        if not self.ethpm_build_dir.exists():
            self.ethpm_build_dir.mkdir()

        copy(self.fixtures_dir / Path("ganache_settings.py"), Path("settings.py"))

    def teardown_method(self, method):
        rmtree(self.tmp_path)

        if Path("settings.py").exists():
            Path("settings.py").unlink()



from json import dumps
from pathlib import Path
from typing import Dict, Tuple
from sys import path
from os import getcwd

import ipfshttpclient

from black_mamba.constants import (SMART_CONTRACT_BUILD_DIR,
                                   SMART_CONTRACT_SOURCE_DIR,
                                   EPM_BUILD_DIR,
                                   MANIFEST_BUILD_FILE)
from black_mamba.contract.build_contract import BuildContract


def ask_meta_from_user() -> Dict:
    print("Please answer these questions for meta!")
    authors_string = input("Authors (separate them by comma), e.g. Arjuna <arjuna@mamba.black>: ")
    authors = list(map(lambda x: x.strip(), authors_string.split(", ")))
    description = input("Description: ")
    keywords_string = input("Keywords (separate them by comma): ")
    keywords = list(map(lambda x: x.strip(), keywords_string.split(", ")))
    license = input("License: ")
    documentation = input("Documentation URL: ")
    repo = input("Repository: ")
    website = input("Website: ")
    return {
        "authors": authors,
        "description": description,
        "keywords": keywords,
        "license": license,
        "links": {
            "documentation": documentation,
            "repo": repo,
            "website": website
        }
    }

def ask_package_version_from_user() -> Tuple:
    print("Please answer these questions for package and version!")
    package = input("Package name: ")
    version = input("Version: ")
    return (package, version)

def create_contract_types(build_contracts_directory : Path = SMART_CONTRACT_BUILD_DIR) -> Dict:
    contract_types_dict = {
    }

    for smart_contract in build_contracts_directory.iterdir():
        if not smart_contract.name.startswith("."):
            contract_type = {}
            build_contract_object = BuildContract(smart_contract.name, build_contracts_directory)
            build_contract = build_contract_object.build_contract
            contract_type["abi"] = build_contract["abi"]
            contract_type["compiler"] = {
                "name": build_contract["compiler"]["name"],
                "settings": {"optimize": False},
                "version": build_contract["compiler"]["version"]
            }
            contract_type["deployment_bytecode"] = {
                "bytecode": build_contract["bytecode"]
            }
            if build_contract["devdoc"]:
                contract_type["natspec"] = {
                    "details": build_contract["devdoc"]["details"],
                    "methods": build_contract["devdoc"]["methods"]
                }
            contract_type["runtime_bytecode"] = {
                "bytecode": build_contract["bytecode_runtime"]
            }
            contract_types_dict[smart_contract.name[:-5]] = contract_type

    return contract_types_dict

def create_sources(package_name: str, source_contracts_directory : Path = SMART_CONTRACT_SOURCE_DIR) -> Dict:
    sources = {
    }

    for smart_contract in source_contracts_directory.iterdir():
        if not smart_contract.name.startswith("."):
            source_key = f"./{package_name}/{smart_contract.name}"
            with open(smart_contract, "r") as f:
                content = f.read()
                sources[source_key] = content

    return sources

def create_manifest(contract_types: Dict, meta: Dict, package_name: str, sources: Dict, version: str) -> Dict:
    manifest = {
        "contract_types": contract_types,
        "manifest_version": "2",
        "meta": meta,
        "package_name": package_name,
        "sources": sources,
        "version": version
    }
    return manifest

def write_manifest(build_contracts_directory : Path = SMART_CONTRACT_BUILD_DIR,
                   source_contracts_directory : Path = SMART_CONTRACT_SOURCE_DIR,
                   manifest_written_file : Path = MANIFEST_BUILD_FILE):
    package_version = ask_package_version_from_user()
    package_name = package_version[0]
    version = package_version[1]
    meta = ask_meta_from_user()
    sources = create_sources(package_name, source_contracts_directory)
    contract_types = create_contract_types(build_contracts_directory)
    manifest = create_manifest(contract_types, meta, package_name, sources, version)
    json_string = dumps(manifest, sort_keys=True, separators=(",", ":"))
    if not EPM_BUILD_DIR.exists():
        EPM_BUILD_DIR.mkdir()
    with open(manifest_written_file, "w") as f:
        f.write(json_string)

def pin_manifest(manifest_written_file : Path = MANIFEST_BUILD_FILE, settings_directory : Path = getcwd()):
    path.append(settings_directory)
    import settings
    ipfs_address = settings.ipfs_settings["address"]

    with ipfshttpclient.connect(ipfs_address) as client:
        pinned_asset = client.add(manifest_written_file, recursive=False)
        print(f"Pinned asset URI is {pinned_asset['Hash']}.")

    return pinned_asset



from pathlib import Path


ETHEREUM_PACKAGES_DIR = "ethpm_packages"
SMART_CONTRACT_BUILD_DIR = Path(".") / Path("build") / Path("contracts")
SMART_CONTRACT_SOURCE_DIR = Path(".") / Path("contracts")
EPM_BUILD_DIR = Path(".") / Path("ethpm_build")
MANIFEST_BUILD_FILE = EPM_BUILD_DIR / Path("manifest.json")



from json import load
from pathlib import Path
from sys import path
from os import getcwd

import pytest
from web3 import (
    EthereumTesterProvider,
    Web3,
)


esp = EthereumTesterProvider()


@pytest.fixture
def eth_tester():
    return esp.ethereum_tester

@pytest.fixture
def w3():
    return get_w3()

def contract(smart_contract_name, parameters=[], contract_directory=Path('.')):
    build_contracts_dir = contract_directory / Path('build') / Path('contracts')
    contract_json_file = (build_contracts_dir / smart_contract_name).with_suffix('.json')

    with open(contract_json_file, 'r') as smart_contract_build_file:
        json_object = load(smart_contract_build_file)
        if json_object["compiler"]["name"] == "solidity":
            bytecode = json_object["bin"]
        else:
            bytecode = json_object["bytecode"]
        abi = json_object["abi"]
        w3 = get_w3()
        contract = w3.eth.contract(abi=abi, bytecode=bytecode)
        tx_hash = contract.constructor(*parameters).transact({"from": w3.eth.accounts[0]})
        tx_receipt = w3.eth.waitForTransactionReceipt(tx_hash)
        contract = w3.eth.contract(abi=abi, address=tx_receipt.contractAddress)
        return contract

def get_w3():
    settings_directory = getcwd()
    path.append(settings_directory)
    import settings
    try:
        ganache_cli = settings.ganache_cli
        address = ganache_cli["development"]["address"]
        w3 = Web3(Web3.HTTPProvider(address))
        if w3.isConnected():
            return w3
        else:
            esp = EthereumTesterProvider()
            return Web3(esp)
    except AttributeError:
        return Web3(esp)


class TestContract:

    def setup_method(self, method):
        w3 = get_w3()
        try:
            w3.provider.ethereum_tester
        except AttributeError:
            w3.testing.snapshot()

    def teardown_method(self, method):
        w3 = get_w3()
        try:
            w3.provider.ethereum_tester
        except AttributeError:
            w3.testing.revert(1)



from json import load, dump, JSONEncoder
from pathlib import Path
from typing import List, Dict, Optional, Any
from os import PathLike, getcwd

from web3 import Web3
from hexbytes import HexBytes
from web3.datastructures import AttributeDict

from black_mamba.contract.contract import Contract
from black_mamba.constants import SMART_CONTRACT_BUILD_DIR


class HexJsonEncoder(JSONEncoder):

    def default(self, obj):
        if isinstance(obj, HexBytes):
            return obj.hex()
        if isinstance(obj, AttributeDict):
            return dict(obj)
        return super().default(obj)

class DeployContract(Contract):

    def __init__(self, settings_directory : Path = getcwd()):
        super().__init__(settings_directory)

    def contract(self,
                 smart_contract_name : str,
                 build_contracts_directory : Path = SMART_CONTRACT_BUILD_DIR):
        contract_json_file = (build_contracts_directory / smart_contract_name).with_suffix('.json')

        with open(contract_json_file, "r") as smart_contract_build_file:
            json_object = load(smart_contract_build_file)
            if json_object["compiler"]["name"] == "solidity":
                bytecode = json_object["bin"]
            else:
                bytecode = json_object["bytecode"]
            abi = json_object["abi"]
            smart_contract = self.w3.eth.contract(abi=abi, bytecode=bytecode)
            return smart_contract

    def deployed_contract(self,
                          smart_contract_name : str,
                          build_contracts_directory : Path = SMART_CONTRACT_BUILD_DIR,
                          deployed_contracts_directory : Path = Path(getcwd()) / Path("deployed")):
        prefix_deployed_file = "receipt_"
        contract_json_file = (build_contracts_directory / smart_contract_name).with_suffix(".json")
        deployed_name = prefix_deployed_file + smart_contract_name
        deployed_json_file = (deployed_contracts_directory / deployed_name).with_suffix(".json")

        with open(contract_json_file, "r") as smart_contract_build_file, open(deployed_json_file, "r") as smart_contract_deployed_file:
            json_object = load(smart_contract_build_file)
            abi = json_object["abi"]
            deployed_json_object = load(smart_contract_deployed_file)
            address = deployed_json_object["contractAddress"]
            smart_contract = self.w3.eth.contract(abi=abi, address=address)
            return smart_contract

    def deploy_contract(self,
                        smart_contract_name : str,
                        parameters : List = [],
                        transaction_parameters : Dict[str, Optional[Any]] = {},
                        private_key : Optional[str] = None,
                        build_contracts_directory : Path = Path(".") / Path("build") / Path("contracts"),
                        deployed_directory : Path = Path(".") / Path("deployed")):

        smart_contract = self.contract(smart_contract_name, build_contracts_directory)
        smart_contract_constructor = smart_contract.constructor(*parameters)
        if private_key:
            from_account = transaction_parameters["from"]
            nonce = self.w3.eth.getTransactionCount(from_account)
            transaction_parameters["nonce"] = nonce
            tx_built = smart_contract_constructor.buildTransaction(transaction_parameters)
            tx_signed = self.w3.eth.account.signTransaction(tx_built, private_key=private_key)
            tx_hash = self.w3.eth.sendRawTransaction(tx_signed.rawTransaction)
        else:
            if transaction_parameters and not transaction_parameters.get("from", None):
                transaction_parameters["from"] = self.w3.geth.personal.listAccounts()[0]
            if transaction_parameters and not transaction_parameters.get("nonce", None):
                from_account = transaction_parameters["from"]
                nonce = self.w3.eth.getTransactionCount(from_account)
                transaction_parameters["nonce"] = nonce
            tx_hash = smart_contract_constructor.transact(transaction_parameters)
        tx_receipt = self.w3.eth.waitForTransactionReceipt(tx_hash)
        tx_dict = dict(tx_receipt)

        receipt_name = "receipt_%s" % smart_contract_name
        receipt_json_file = (deployed_directory / receipt_name).with_suffix('.json')

        with open(receipt_json_file, "w") as json_write_file:
            dump(tx_dict, json_write_file, cls=HexJsonEncoder, indent=4)



from pathlib import Path
from json import load

from black_mamba.constants import SMART_CONTRACT_BUILD_DIR


class BuildContract:

    build_contract = {}

    def __init__(self,
                 smart_contract_name : str,
                 build_contracts_directory : Path = SMART_CONTRACT_BUILD_DIR):
        contract_json_file = (build_contracts_directory / smart_contract_name).with_suffix('.json')

        with open(contract_json_file, "r") as smart_contract_build_file:
            self.build_contract = load(smart_contract_build_file)



import os


networks = {
    "development": {
        "mode": "HTTP",
        "host": "localhost",
        "port": 7545,
        "network_id": "*"
    }

    #"development": {
    #    "mode": "IPC",
    #    "url": "/home/sarahconnor/ethereum/data/geth.ipc"
    #}

    #"development": {
    #    "mode": "Websocket",
    #    "host": "localhost",
    #    "port": 8546,
    #    "network_id": "*"
    #}

    #"development": {
    #    "mode": "IPC",
    #    "url": "/home/sarahconnor/ethereum/data/geth.ipc"
    #}

    #"development": {
    #    "mode": "Infura",
    #    "scheme": "https / wss",
    #    "endpoints": "mainnet / ropsten / goerly / rinkeby / kovan"
    #}
}

ipfs_settings = {
    # Choose one
    "address": "https://ipfs.io/ipfs/",                # IPFS Gateway
    "address": "/dns4/ipfs.infura.io/tcp/5001/https/", # Infura
    "address": "/ip4/127.0.0.1/tcp/5001"               # Local IPFS node
}

infura_settings = {
    "project_id": "FILL_IT_WITH_PROJECT_ID",
    "project_secret": "FILL_IT_WITH_PROJECT_SECRET",
}

auth = {
    "development": {
        #"private_key": os.environ("MY_PRIVATE_KEY")
        "password": "password_to_unlock_keyfile",
        "keyfile": "keyfile.json"
    }
}

ganache_cli = {
    "development": {
        "address": "http://127.0.0.1:8545"
    }
}

etherscan = {
    "development": {
        "api_key": "ETHERSCAN_API_KEY"
    }
}



import requests

from black_mamba import settings_directory
import settings

etherscan = settings.etherscan

api_key = etherscan["development"]["api_key"]

server = 'https://api.etherscan.io'


def ether_balance_multi(account_addresses):
    """Getting the balances of ETH of accounts

    Parameters
    ----------
    account_addresses : array of str
      The addresses of the accounts
    """
    account_addresses_string = ",".join(account_addresses)
    response = requests.get(f'{server}/api?module=account&action=balancemulti&address={account_addresses_string}&tag=latest&apikey={api_key}')
    if response.ok:
        return response.json()["result"]

def ether_balance_single(account_address):
    """Getting the balance of ETH of an account

    Parameters
    ----------
    account_address : str
      The address of the account
    """
    response = requests.get(f'{server}/api?module=account&action=balance&address={account_address}&tag=latest&apikey={api_key}')
    if response.ok:
        return response.json()["result"]

def normal_transactions(account_address, startblock=0, endblock=99999999, sort="asc"):
    """Getting the list of normal transactions

    Parameters
    ----------
    account_address : str
      The address of the account
    startblock : int, optional
      The start of the block number (default is 0)
    endblock : int, optional
      The end of the block number (default is 99999999)
    sort : str, optional
      The way of the result is sorted, "asc" or "desc" (default is "asc")
    """
    response = requests.get(f'{server}/api?module=account&action=txlist&address={account_address}&startblock={startblock}&endblock={endblock}&sort={sort}&apikey={api_key}')
    if response.ok:
        return response.json()["result"]

def normal_transactions_paginated(account_address, startblock=0, endblock=99999999, page=1, offset=10, sort="asc"):
    """Getting the list of normal transactions (pagination version)

    Parameters
    ----------
    account_address : str
      The address of the account
    startblock : int, optional
      The start of the block number (default is 0)
    endblock : int, optional
      The end of the block number (default is 99999999)
    page : int, optional
      The page number (default is 1)
    offset : int, optional
      Maximum records to be returned (default is 10)
    sort : str, optional
      The way of the result is sorted, "asc" or "desc" (default is "asc")
    """
    response = requests.get(f'{server}/api?module=account&action=txlist&address={account_address}&startblock={startblock}&endblock={endblock}&page={page}&offset={offset}&sort={sort}&apikey={api_key}')
    if response.ok:
        return response.json()["result"]

def internal_transactions(account_address, startblock=0, endblock=99999999, sort="asc"):
    """Getting the list of internal transactions

    Parameters
    ----------
    account_address : str
      The address of the account
    startblock : int, optional
      The start of the block number (default is 0)
    endblock : int, optional
      The end of the block number (default is 99999999)
    sort : str, optional
      The way of the result is sorted, "asc" or "desc" (default is "asc")
    """
    response = requests.get(f'{server}/api?module=account&action=txlistinternal&address={account_address}&startblock={startblock}&endblock={endblock}&sort={sort}&apikey={api_key}')
    if response.ok:
        return response.json()["result"]

def internal_transactions_paginated(account_address, startblock=0, endblock=99999999, page=1, offset=10, sort="asc"):
    """Getting the list of internal transactions (pagination version)

    Parameters
    ----------
    account_address : str
      The address of the account
    startblock : int, optional
      The start of the block number (default is 0)
    endblock : int, optional
      The end of the block number (default is 99999999)
    page : int, optional
      The page number (default is 1)
    offset : int, optional
      Maximum records to be returned (default is 10)
    sort : str, optional
      The way of the result is sorted, "asc" or "desc" (default is "asc")
    """
    response = requests.get(f'{server}/api?module=account&action=txlistinternal&address={account_address}&startblock={startblock}&endblock={endblock}&page={page}&offset={offset}&sort={sort}&apikey={api_key}')
    if response.ok:
        return response.json()["result"]

def internal_transactions_transaction_hash(txhash):
    """Getting the list of internal transactions by transaction hash

    Parameters
    ----------
    txhash : str
      The transaction hash
    """
    response = requests.get(f'{server}/api?module=account&action=txlistinternal&txhash={txhash}&apikey={api_key}')
    if response.ok:
        return response.json()["result"]

def internal_transactions_block_range(startblock=0, endblock=99999999, page=1, offset=10, sort="asc"):
    """Getting the list of internal transactions by block range

    Parameters
    ----------
    startblock : int, optional
      The start of the block number (default is 0)
    endblock : int, optional
      The end of the block number (default is 99999999)
    page : int, optional
      The page number (default is 1)
    offset : int, optional
      Maximum records to be returned (default is 10)
    sort : str, optional
      The way of the result is sorted, "asc" or "desc" (default is "asc")
    """
    response = requests.get(f'{server}/api?module=account&action=txlistinternal&startblock={startblock}&endblock={endblock}&page={page}&offset={offset}&sort={sort}&apikey={api_key}')
    if response.ok:
        return response.json()["result"]

def erc20_token_transfer_events(account_address, startblock=0, endblock=99999999, sort="asc"):
    """Getting the list of ERC-20 token transfer events

    Parameters
    ----------
    account_address : str
      The address of the account
    startblock : int, optional
      The start of the block number (default is 0)
    endblock : int, optional
      The end of the block number (default is 99999999)
    sort : str, optional
      The way of the result is sorted, "asc" or "desc" (default is "asc")
    """
    response = requests.get(f'{server}/api?module=account&action=tokentx&address={account_address}&startblock={startblock}&endblock={endblock}&sort={sort}&apikey={api_key}')
    if response.ok:
        return response.json()["result"]

def erc20_token_transfer_events_paginated(account_address, startblock=0, endblock=99999999, page=1, offset=10, sort="asc"):
    """Getting the list of ERC-20 token transfer events (pagination version)

    Parameters
    ----------
    account_address : str
      The address of the account
    startblock : int, optional
      The start of the block number (default is 0)
    endblock : int, optional
      The end of the block number (default is 99999999)
    page : int, optional
      The page number (default is 1)
    offset : int, optional
      Maximum records to be returned (default is 10)
    sort : str, optional
      The way of the result is sorted, "asc" or "desc" (default is "asc")
    """
    response = requests.get(f'{server}/api?module=account&action=tokentx&address={account_address}&startblock={startblock}&endblock={endblock}&page={page}&offset={offset}&sort={sort}&apikey={api_key}')
    if response.ok:
        return response.json()["result"]

def erc20_token_transfer_events_contract_address(contract_address, account_address, page=1, offset=10, sort="asc"):
    """Getting the list of ERC-20 token transfer events with a smart contract's address

    Parameters
    ----------
    contract_address : str
      The address of the ERC-20 token smart contract
    account_address : str
      The address of the account
    page : int, optional
      The page number (default is 1)
    offset : int, optional
      Maximum records to be returned (default is 10)
    sort : str, optional
      The way of the result is sorted, "asc" or "desc" (default is "asc")
    """
    response = requests.get(f'{server}/api?module=account&action=tokentx&contractaddress={contract_address}&address={account_address}&page={page}&offset={offset}&sort={sort}&apikey={api_key}')
    if response.ok:
        return response.json()["result"]

def erc721_token_transfer_events(account_address, startblock=0, endblock=99999999, sort="asc"):
    """Getting the list of ERC-721 token transfer events

    Parameters
    ----------
    account_address : str
      The address of the account
    startblock : int, optional
      The start of the block number (default is 0)
    endblock : int, optional
      The end of the block number (default is 99999999)
    sort : str, optional
      The way of the result is sorted, "asc" or "desc" (default is "asc")
    """
    response = requests.get(f'{server}/api?module=account&action=tokennfttx&address={account_address}&startblock={startblock}&endblock={endblock}&sort={sort}&apikey={api_key}')
    if response.ok:
        return response.json()["result"]

def erc721_token_transfer_events_paginated(account_address, startblock=0, endblock=99999999, page=1, offset=10, sort="asc"):
    """Getting the list of ERC-721 token transfer events (pagination version)

    Parameters
    ----------
    account_address : str
      The address of the account
    startblock : int, optional
      The start of the block number (default is 0)
    endblock : int, optional
      The end of the block number (default is 99999999)
    page : int, optional
      The page number (default is 1)
    offset : int, optional
      Maximum records to be returned (default is 10)
    sort : str, optional
      The way of the result is sorted, "asc" or "desc" (default is "asc")
    """
    response = requests.get(f'{server}/api?module=account&action=tokennfttx&address={account_address}&startblock={startblock}&endblock={endblock}&page={page}&offset={offset}&sort={sort}&apikey={api_key}')
    if response.ok:
        return response.json()["result"]

def erc721_token_transfer_events_contract_address(contract_address, account_address, page=1, offset=10, sort="asc"):
    """Getting the list of ERC-721 token transfer events with a smart contract's address

    Parameters
    ----------
    contract_address : str
      The address of the ERC-721 token smart contract
    account_address : str
      The address of the account
    page : int, optional
      The page number (default is 1)
    offset : int, optional
      Maximum records to be returned (default is 10)
    sort : str, optional
      The way of the result is sorted, "asc" or "desc" (default is "asc")
    """
    response = requests.get(f'{server}/api?module=account&action=tokennfttx&contractaddress={contract_address}&address={account_address}&page={page}&offset={offset}&sort={sort}&apikey={api_key}')
    if response.ok:
        return response.json()["result"]

def blocks_mined(account_address):
    """Getting the list of block mined by an account

    Parameters
    ----------
    account_address : str
      The address of the account
    """
    response = requests.get(f'{server}/api?module=account&action=getminedblocks&address={account_address}&blocktype=blocks&apikey={api_key}')
    if response.ok:
        return response.json()["result"]

def blocks_mined_paginated(account_address, page=1, offset=10):
    """Getting the list of block mined by an account (pagination version)

    Parameters
    ----------
    account_address : str
      The address of the account
    page : int, optional
      The page number (default is 1)
    offset : int, optional
      Maximum records to be returned (default is 10)
    """
    response = requests.get(f'{server}/api?module=account&action=getminedblocks&address={account_address}&blocktype=blocks&page={page}&offset={offset}&apikey={api_key}')
    if response.ok:
        return response.json()["result"]



from argparse import ArgumentParser
from pathlib import Path
from subprocess import run

from black_mamba.initialization.init import initialize_project_directory
from black_mamba.compilation.vyper_compiler import compile_all_files
from black_mamba.server.geth_dev_server import run_server
from black_mamba.auth import Authentication
from black_mamba.mnemonic import Mnemonic
from black_mamba.epm.package_manager import PackageManager
from black_mamba.constants import ETHEREUM_PACKAGES_DIR


def parse_cli_args():

    parser = ArgumentParser(description="Mamba Framework")
    parser.add_argument('mode', type=str, help="Mode of Mamba CLI: init / compile / server / keyfile")
    parser.add_argument('--httpserver', action="store_true", default=False)
    parser.add_argument('--websocket', action="store_true", default=False)

    keyfile_group = parser.add_argument_group('keyfile')
    keyfile_group.add_argument('--keyfile_mode', action="store", help="encrypt or decrypt the keyfile", default=False)
    keyfile_group.add_argument('--keyfile_file', action="store", default=False)
    keyfile_group.add_argument('--keyfile_private_key', action="store", default=False)
    keyfile_group.add_argument('--keyfile_password', action="store", default=False)

    epm_group = parser.add_argument_group('epm')
    epm_help = """Mode of Ethereum package manager: install / list / uninstall / create.
    "install" -> Install package from GitHub or registry.
    "list" -> List all installed packages.
    "uninstall" -> Uninstall package.
    "create" -> Create a package manifest from the source code, ABI, and bytecode of the smart contract.
    "pin" -> Pin the package manifest to IPFS.
    """
    epm_group.add_argument('--epm_mode', action="store", help=epm_help, default="list")
    epm_group.add_argument('--epm_uri', action="store", default=None)
    epm_group.add_argument('--epm_package', action="store", default=None)

    mnemonic_group = parser.add_argument_group('generate_mnemonic')
    mnemonic_group.add_argument('--num_words', action="store", help="number of words in mnemonic (must be 12, 15, 18, 21, 24)", default=12)
    mnemonic_group.add_argument('--language', action="store", help="language to use (must be english, czech, french, italian, japanese, korean, spanish, chinese_simplified, chinese_traditional)", default="english")

    test_group = parser.add_argument_group('test')
    test_group.add_argument('--file', action="store", default=None)

    arguments = parser.parse_args()

    if arguments.mode=="init":
        initialize_project_directory(Path('.'))
    elif arguments.mode=="compile":
        compile_all_files(Path('contracts'), Path('build') / Path('contracts'), Path("migrations"))
    elif arguments.mode=="server":
        run_server(Path("datadir"), arguments.httpserver, arguments.websocket)
    elif arguments.mode=="keyfile":
        if arguments.keyfile_mode == "encrypt":
            Authentication.encrypt_keyfile(arguments.keyfile_file, arguments.keyfile_private_key, arguments.keyfile_password)
        elif arguments.keyfile_mode == "decrypt":
            Authentication.decrypt_keyfile(arguments.keyfile_file, arguments.keyfile_password)
    elif arguments.mode=="epm":
        epm = PackageManager(Path(ETHEREUM_PACKAGES_DIR))
        epm.operate(arguments.epm_mode, arguments.epm_uri, arguments.epm_package)
    elif arguments.mode=="generate_mnemonic":
        mnemonic = Mnemonic.generate_mnemonic(int(arguments.num_words), arguments.language)
        print(mnemonic)
    elif arguments.mode=="test":
        pytest_args = arguments.file.split(" ")
        run(["py.test", *pytest_args])


if __name__ == "__main__":
    parse_cli_args()



from sys import path
from pathlib import Path
from os import getcwd, environ

from web3 import Web3


class Contract:

    def __init__(self, settings_directory : Path = getcwd()):
        path.append(settings_directory)
        import settings
        networks = settings.networks

        infura_settings = settings.infura_settings
        if infura_settings:
            environ["WEB3_INFURA_PROJECT_ID"] = infura_settings["project_id"]
            environ["WEB3_INFURA_API_SECRET"] = infura_settings["project_secret"]

        development_network = networks["development"]
        if development_network["mode"]=="HTTP":
            server = "http://" + development_network["host"] + ":" + str(development_network["port"])
            self.w3 = Web3(Web3.HTTPProvider(server))
        elif development_network["mode"]=="IPC":
            ipc_url = development_network["url"]
            self.w3 = Web3(Web3.IPCProvider(ipc_url))
        elif development_network["mode"]=="Websocket":
            server = "ws://" + development_network["host"] + ":" + str(development_network["port"])
            self.w3 = Web3(Web3.WebsocketProvider(server))
        elif development_network["mode"]=="Infura":
            environ["WEB3_INFURA_SCHEME"] = str(development_network["scheme"])
            if development_network["endpoints"] == "mainnet":
                from web3.auto.infura.mainnet import w3
            elif development_network["endpoints"] == "ropsten":
                from web3.auto.infura.ropsten import w3
            elif development_network["endpoints"] == "goerli":
                from web3.auto.infura.goerli import w3
            elif development_network["endpoints"] == "rinkeby":
                from web3.auto.infura.rinkeby import w3
            elif development_network["endpoints"] == "kovan":
                from web3.auto.infura.kovan import w3
            self.w3 = w3



from pathlib import Path
from shutil import copyfile


def initialize_project_directory(current_directory: Path):
    contracts_dir = current_directory / Path('contracts')
    contracts_dir.mkdir()

    build_dir = current_directory / Path('build')
    build_dir.mkdir()
    build_contracts_dir = current_directory / Path('build') / Path('contracts')
    build_contracts_dir.mkdir()

    test_dir = current_directory / Path('test')
    test_dir.mkdir()

    migrations_dir = current_directory / Path("migrations")
    migrations_dir.mkdir()

    deployed_dir = current_directory / Path('deployed')
    deployed_dir.mkdir()

    decentralized_app_dir = current_directory / Path('decentralized_app')
    decentralized_app_dir.mkdir()

    settings_sample = Path(__file__).parent / Path("init_files") / Path("settings_sample.py")
    settings_file = current_directory / Path("settings.py")
    copyfile(settings_sample, settings_file)



from json import dumps, loads
from pathlib import Path
from shutil import rmtree
from os import getcwd

from typing import List

from ethpm import Package
from black_mamba.contract.contract import Contract
from black_mamba.epm.manifest import write_manifest, pin_manifest


class PackageManager:

    def __init__(self, packages_dir: Path = Path(getcwd()) / Path("ethpm_packages")):
        """
        A package manager's purpose is to install, remove, list packages from chain or files.
        """
        self.packages_dir = packages_dir
        self.w3 = None

    def operate(self, mode: str, uri: str, package: str):
        """
        Convenient method called from Mamba CLI.
        """
        if mode=="install":
            self.install(uri)
        elif mode=="list":
            self.list()
        elif mode=="uninstall":
            self.uninstall(package)
        elif mode=="create":
            write_manifest()
        elif mode=="pin":
            pin_manifest()

    def install(self, uri: str):
        """
        Install packages from chain or files and put it in packages directory.
        """
        self._create_ethpm_packages_dir()
        self._initialize_w3()

        is_file = True
        for prefix in ("ipfs://", "https://", "ethpm://", "erc1319://"):
            if uri.startswith(prefix):
                is_file = False
                break

        if is_file:
            package = Package.from_file(Path(uri), self.w3)
        else:
            package = Package.from_uri(uri, self.w3)

        self._write_manifests_to_filesystem(package.name, package.version, package.manifest)
        self.package = package

    def list(self):
        """
        List installed packages.
        """
        packages_list = list(map(lambda x: x.name, self.packages_dir.iterdir()))
        print(*packages_list, sep="\n")

    def uninstall(self, package_delete: str):
        """
        Remove an installed package.
        """
        for package_dir in self.packages_dir.iterdir():
            if package_dir.name == package_delete:
                rmtree(package_dir)
                print(f"Deleted {package_delete}")

    def load(self, contract_factory: str, package: Path, version: str = ""):
        """
        Load an installed package.
        """
        package_path = self.packages_dir / Path(package)
        versions = list(map(lambda p: p.name, package_path.iterdir()))
        if not version:
            version = self.find_max_version(versions)
        manifest_path = package_path / Path(version) / Path("manifest.json")

        self._initialize_w3()

        self.package = Package.from_file(manifest_path, self.w3)
        factory = self.package.get_contract_factory(contract_factory)

        factory.web3 = self.w3

        return factory

    def get_abi(self, contract_factory: str, package: Path, version: str = ""):
        """
        Get abi from the manifest file
        """
        package_path = self.packages_dir / Path(package)
        versions = list(map(lambda p: p.name, package_path.iterdir()))
        if not version:
            version = self.find_max_version(versions)
        manifest_path = package_path / Path(version) / Path("manifest.json")

        with open(manifest_path) as f:
            content = f.read()
        manifest = loads(content)
        abi = manifest["contract_types"][contract_factory]["abi"]

        return abi

    def _create_ethpm_packages_dir(self):
        if not self.packages_dir.exists():
            self.packages_dir.mkdir()

    def _write_manifests_to_filesystem(self, name: str, version: str, manifest: str):
        package_dir = self.packages_dir / Path(name)
        if not package_dir.exists():
            package_dir.mkdir()
        version_dir = package_dir / Path(version)
        if not version_dir.exists():
            version_dir.mkdir()
        manifest_content = dumps(manifest, sort_keys=True, separators=(",", ":"))
        manifest_file = version_dir / Path("manifest.json")
        with open(manifest_file, "w") as f:
            f.write(manifest_content)

    def _initialize_w3(self):
        contract_instance = Contract()
        self.w3 = contract_instance.w3

    def find_max_version(self, versions: List) -> str:
        """
        Given ["1.2.3", "3.2.11", "3.2.7", "2.0.0"], this method would return "3.2.11"
        """
        maximum = None

        def conv_to_int_arr(v):
            return list(map(lambda x: int(x), v.split(".")))

        if len(versions) == 0:
            return ""

        for version in versions:
            int_version = conv_to_int_arr(version)

            if len(int_version) == 1:
                int_version = [int_version[0], 0, 0]
            elif len(int_version) == 2:
                int_version = [int_version[0], int_version[1], 0]

            if maximum == None:
                maximum = int_version
            else:
                if maximum[0] > int_version[0]:
                    continue
                elif maximum[1] > int_version[1]:
                    continue
                elif maximum[2] > int_version[2]:
                    continue
                else:
                    maximum = int_version

        max_version = ".".join(map(lambda x: str(x), maximum))
        return max_version



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['black_mamba',
 'black_mamba.auth',
 'black_mamba.compilation',
 'black_mamba.compilation.compiled_files',
 'black_mamba.contract',
 'black_mamba.deploy',
 'black_mamba.epm',
 'black_mamba.etherscan',
 'black_mamba.initialization',
 'black_mamba.initialization.init_files',
 'black_mamba.mnemonic',
 'black_mamba.server',
 'black_mamba.testlib']

package_data = \
{'': ['*'], 'black_mamba.server': ['server_files/*']}

install_requires = \
['eth-tester==0.5.0b4',
 'py-evm==0.4.0a4',
 'pytest-mock==3.5.1',
 'pytest==6.2.1',
 'vyper==0.2.12',
 'web3==5.19.0']

entry_points = \
{'console_scripts': ['mamba = black_mamba.mamba_cli:parse_cli_args']}

setup_kwargs = {
    'name': 'black-mamba',
    'version': '0.6.4',
    'description': 'Development framework to write, test and deploy smart contracts written in Vyper and Solidity. It has integrated web3.py support.',
    'long_description': '# Mamba\n\nMamba is a framework to write, compile, and deploy smart contracts\nwritten in Vyper language and Solidity language. On top of that, it has supports for writing\nand testing decentralized applications using Web3.py and Pytest.\n\n## Dependencies\n\n* [Ganache](https://www.trufflesuite.com/ganache)\n* [Ganache CLI](https://github.com/trufflesuite/ganache-cli)\n* [Pip](https://pypi.org/project/pip/)\n* [Python3](https://www.python.org/downloads/) version 3.6 or greater\n* [Go Ethereum](https://geth.ethereum.org/downloads/)\n* [Vyper](https://github.com/ethereum/vyper)\n* [Solidity](https://soliditylang.org)\n\n## Installation\n\nYou can install the latest release via ``pip``:\n\n```bash\n$ pip install black-mamba\n```\n\nTo use Solidity, you must install ``solc`` compiler separately.\n\n## Quick Usage\n\nTo set up the the structure of Mamba project directory:\n\n```bash\n$ mamba init\n$ edit contracts/HelloWorld.vy\n$ mamba compile\n```\n\n## Documentation\n\nMamba documentation can be found at [Mamba website](https://mamba.black/documentation).\n\n\n## Contributing\n\nI invite you to join Mamba squad! You can contribute to Mamba by\nwriting documentation, finding bugs, and creating pull requests.\n',
    'author': 'Arjuna Sky Kok',
    'author_email': 'arjuna@mamba.black',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://mamba.black',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.6,<4.0',
}


setup(**setup_kwargs)



from eth_keyfile import extract_key_from_keyfile, create_keyfile_json
from eth_account import hdaccount, account
from eth_utils.hexadecimal import encode_hex
import getpass
from codecs import encode, decode
from json import dumps


class Authentication:

    @staticmethod
    def get_private_key_from_keyfile_by_explicit_password(file: str, password: str) -> bytes:
        private_key = extract_key_from_keyfile(file, password.encode())
        encoded = encode(private_key, "hex")
        decoded_private_key = encoded.decode()
        return decoded_private_key

    @staticmethod
    def get_private_key_from_keyfile_by_asking_password(file: str) -> bytes:
        password = getpass.getpass("Password: ")
        private_key = extract_key_from_keyfile(file, password.encode())
        encoded = encode(private_key, "hex")
        decoded_private_key = encoded.decode()
        return decoded_private_key

    @staticmethod
    def encrypt_keyfile(file: str, private_key: str, password: str):
        encoded_private_key = decode(private_key.encode(), "hex")
        output_dict = create_keyfile_json(encoded_private_key, password.encode())
        output_str = dumps(output_dict)
        output_double_quoted = output_str.replace("'", '"') # Replace single quote to double quote
        with open(file, "w") as f:
            f.write(output_double_quoted)
        print(f"{file} is created")

    @staticmethod
    def decrypt_keyfile(file: str, password: str):
        private_key = extract_key_from_keyfile(file, password.encode())
        encoded = encode(private_key, "hex")
        decoded_private_key = encoded.decode()
        print(f"{decoded_private_key}")

    @staticmethod
    def get_private_key_from_mnemonic(mnemonic: str, passphrase: str = "", hdpath: str=hdaccount.ETHEREUM_DEFAULT_PATH) -> bytes:
        seed = account.seed_from_mnemonic(mnemonic, passphrase)
        key = account.key_from_seed(seed, hdpath)
        hex_key = encode_hex(key)
        return hex_key[2:]



from pathlib import Path
from json import dump, loads
import subprocess
from vyper import compile_codes, __version__ as version


def compile_all_files(source_code_directory: Path, build_directory: Path, migration_directory: Path):
    vyper_files = source_code_directory.glob("*.vy")
    for vyper_file in vyper_files:
        with open(vyper_file, 'r') as f:
            build_dir_str = str(build_directory)
            smart_contract_name = Path(vyper_file).with_suffix('').name
            content = f.read()

            smart_contract = {}
            smart_contract[build_dir_str] = content

            formats = ["abi", "bytecode", "ast_dict", "external_interface", "interface", "method_identifiers", "asm", "source_map",
                      "bytecode_runtime", "opcodes", "opcodes_runtime", "devdoc", "userdoc"]
            compiled_code = compile_codes(smart_contract, formats, 'dict')

            smart_contract_json = {
                "contractName": smart_contract_name,
                "compiler": { "name": "vyper",
                              "version": version
                            }
            }

            for format in formats:
                smart_contract_json[format] = compiled_code[build_dir_str][format]

            contract_json_file = build_dir_str + '/' + smart_contract_name + '.json'
            with open(contract_json_file, 'w') as smart_contract_build_file:
                dump(smart_contract_json, smart_contract_build_file, indent=4)

            migration_sample = Path(__file__).parent / Path("compiled_files") / Path("migration_sample.py")
            prefix_deploy_file = "deploy_"
            migration_file_name = f"{prefix_deploy_file}{smart_contract_name}.py"
            migration_file = migration_directory / Path(migration_file_name)
            with open(migration_file, "w") as file:
                migration_sample_content = migration_sample.read_text()
                content = migration_sample_content.format(smart_contract_name=smart_contract_name)
                file.write(content)

    solidity_files = source_code_directory.glob("*.sol")
    for solidity_file in solidity_files:
        with open(solidity_file, 'r') as f:
            build_dir_str = str(build_directory)
            smart_contract_name = Path(solidity_file).with_suffix('').name
            option_str = "abi,asm,ast,bin,bin-runtime,compact-format,devdoc,generated-sources,generated-sources-runtime,hashes,interface,metadata,opcodes,srcmap,srcmap-runtime,storage-layout,userdoc"
            try:
                p = subprocess.Popen(["solc", "--combined-json", option_str, solidity_file], stdout=subprocess.PIPE)
                out, err = p.communicate()
                out_str = out.decode()
                output_json = loads(out_str)
                for contract in output_json["contracts"]:
                    contract_compilation = output_json["contracts"][contract]
                    compiler_version = output_json["version"]
                    smart_contract_name = contract.split(":")[1]
                    smart_contract_json = {
                        "contractName": smart_contract_name,
                        "compiler": { "name": "solidity",
                                      "version": compiler_version
                                    }
                    }
                    for format in contract_compilation:
                        smart_contract_json[format] = contract_compilation[format]

                    contract_json_file = build_dir_str + '/' + smart_contract_name + '.json'
                    with open(contract_json_file, 'w') as smart_contract_build_file:
                        dump(smart_contract_json, smart_contract_build_file, indent=4)

                    migration_sample = Path(__file__).parent / Path("compiled_files") / Path("migration_sample.py")
                    prefix_deploy_file = "deploy_"
                    migration_file_name = f"{prefix_deploy_file}{smart_contract_name}.py"
                    migration_file = migration_directory / Path(migration_file_name)
                    with open(migration_file, "w") as file:
                        migration_sample_content = migration_sample.read_text()
                        content = migration_sample_content.format(smart_contract_name=smart_contract_name)
                        file.write(content)

            except FileNotFoundError:
                print("You must install Solidity compiler first.")
